#!/bin/bash

# Copyright (C) since 2014 NTT DATA Corporation
#
# This program is free software; you can redistribute it and/or
# Modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details

# bash configuration
SCRIPT_DIR=$(cd $(dirname $0);pwd)
PROG=`basename $0`

########################################
#  Local Variable
########################################

# script version
SCRIPT_VERSION="1.0.0"

# profile name
PROFILE=""

# initial lookup days
DAYS=30

# cert root directory
CERT_ROOT_DIR=${SCRIPT_DIR}/cert

# log directory
LOG_ROOT_DIR=/var/log/cloudn/operationlog

# temporally directory
TMP_ROOT_DIR=/var/log/cloudn/operationlog/.tmp

# last modified date chek file
LAST_MODIFIED_DATE_FILE=LastModifiedDate.txt

CLOUDMONKEY="cloudmonkey"

# xml convert module
LOG_XML_PARSER="${SCRIPT_DIR}/HinemosCloudnComputeLog.py"

# log file
LOG_FILE="CloudnOperationLog.log"

########################################
# Function
########################################

#
# Message Logging
#
function Logging() {
	[ "x${QUIET}" != "xtrue" ] && echo "$1"
	/usr/bin/logger "${PROG} $1"
}

#
# show usage
#
function usage {
	echo "usage : ${PROG} -p profile -q [-d DAYS] [--version] [--help]"
	echo "options : "
	echo "  -p profile  set a profile (profile is same as directory name : /root/cloud/cloudn/cert/<profile>/)"
	echo "  -q          quiet mode"
	echo "  -d days     set period days of initial lookup [default:30]"
	echo "  --version   show version"
	echo "  --help      show usage"
}

########################################
# SHELL
########################################

# help check
for OPT in $@
do
	case $OPT in
		--version)
			echo ${SCRIPT_VERSION}
			exit 0
			;;
		--help)
			usage
			exit 0
			;;
	esac
done

# check parameters
while getopts p:d:q OPT
do
	case $OPT in
		p)
			PROFILE=${OPTARG}
			;;
		d)
			OPTION_D=true
			DAYS=${OPTARG}
			;;
		q)
			QUIET=true
			;;
		*)
			usage
			exit 1
			;;
	esac
done

shift $((OPTIND - 1))

if [ "x${PROFILE}" == "x" ];then
	echo "option (-p) is not specified."
	usage
	exit 1
fi

if [ "x${DAYS}" == "x" ] || [ `expr "${DAYS}" : '[0-9]*' > /dev/null` ]; then
	echo "option (-d) must be number."
	usage
	exit 1
fi

CERT_DIR=${CERT_ROOT_DIR}/${PROFILE}
[ -e ${CERT_DIR} ] || { Logging "certification directory \"${CERT_DIR}\" not found."; exit 1; }

[ -e ${CERT_DIR}/config ] || { Logging "profile config file \"${CERT_DIR}/config\" not found"; exit 1; }

# mkdir
LOG_DIR=${LOG_ROOT_DIR}/${PROFILE}
if [ ! -e ${LOG_DIR} ]; then
	Logging "creating log directory \"${LOG_DIR}\"."
	mkdir -p ${LOG_DIR} || { Logging "failed making directory \"${LOG_DIR}\""; exit 1; }
fi

TMP_DIR=${TMP_ROOT_DIR}/${PROFILE}
if [ ! -e ${TMP_DIR} ]; then
	mkdir -p ${TMP_DIR} || { Logging "failed making directory \"${TMP_DIR}\""; exit 1; }
fi

# [WARNING] Timezone specification('Asia/Tokyo') is required for correct query
if [ ! -e ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE} ]; then
  LAST_MODIFIED_DATE=`TZ='Asia/Tokyo' date -d '-'${DAYS}' day' +"%Y-%m-%d %H:%M:%S"`
else
  LAST_MODIFIED_DATE=`cat ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}`
fi

NOW=`TZ='Asia/Tokyo' date +"%Y-%m-%d %H:%M:%S"`

# query and parse
Logging "downloading logs between \"${LAST_MODIFIED_DATE}\" and \"${NOW}\"..."
FILE_TS=`date +"%Y%m%d%H%M%S"`
WORK_OUTPUT=${TMP_DIR}/cloudmonkey_${FILE_TS}.json
${CLOUDMONKEY} -c ${CERT_DIR}/config "list events startdate=\"${LAST_MODIFIED_DATE}\" enddate=\"${NOW}\"" > ${WORK_OUTPUT}

# remove ANSI escape char and filter 0x000a
sed -i '1 s/^\x1b\[?1034h//;/^$/d' ${WORK_OUTPUT} 

# convert json
if [ -s ${WORK_OUTPUT} ]; then
	${LOG_XML_PARSER} -i ${WORK_OUTPUT} -o ${LOG_DIR}/${LOG_FILE}
fi

echo ${NOW} > ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE} || { Logging "failed save last timestamp to \"${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}\""; exit 1; }

########################################
# Termination Processing
########################################

rm -rf ${TMP_DIR} || { Logging "failed delete temporally directory \"${TMP_DIR}\""; exit 1; }

Logging "updated files at ${NOW}"
Logging "   log file : ${LOG_DIR}/${LOG_FILE}"
Logging "   timestamp file : ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}"

exit 0

